import logging
from typing import Union
from urllib.parse import urljoin

from .types import PaddleJsonType

log = logging.getLogger(__name__)


def refund_payment(
    self,
    order_id: Union[str, int],
    amount: float = None,
    reason: str = None
) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/payments/refundpayment
    """
    url = urljoin(self.vendors_v2, 'payment/refund')

    json = {
        'order_id': order_id,
        'amount': amount,
        'reason': reason,
    }  # type: PaddleJsonType
    return self.post(url=url, json=json)



import logging
from typing import Dict, Union
from urllib.parse import urljoin

log = logging.getLogger(__name__)


def get_user_history(
    self,
    email: str,
    vendor_id: int = None,
    product_id: int = None
) -> dict:
    """
    https://developer.paddle.com/api-reference/checkout-api/user-history/getuserhistory
    """
    url = urljoin(self.checkout_v2, 'user/history')

    params = {'email': email}   # type: Dict[str, Union[str, int]]
    if product_id:
        params['product_id'] = product_id

    if vendor_id:
        params['vendor_id'] = str(vendor_id)
    else:
        params['vendor_id'] = str(self.vendor_id)

    return self.get(url=url, params=params)



import logging
from typing import Dict
from urllib.parse import urljoin

from .constants import supported_countries

log = logging.getLogger(__name__)


def get_prices(
    self,
    product_ids: list,
    customer_country: str = None,
    customer_ip: str = None,
    coupons: list = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/checkout-api/prices/getprices
    """
    url = urljoin(self.checkout_v2, 'prices')

    params = {}  # type: Dict[str, str]
    if product_ids:
        products = ','.join([str(int(product)) for product in product_ids])
        params['product_ids'] = products
    if customer_country:
        if customer_country not in supported_countries.keys():
            raise ValueError('Country code "{0}" is not valid'.format(customer_country))  # NOQA: E501
        params['customer_country'] = customer_country
    if customer_ip:
        params['customer_ip'] = customer_ip
    if coupons:
        params['coupons'] = ','.join(coupons)

    return self.get(url=url, params=params)



import logging
from urllib.parse import urljoin

from .types import DatetimeType, PaddleJsonType
from .validators import validate_date

log = logging.getLogger(__name__)


def generate_license(
    self,
    product_id: int,
    allowed_uses: int,
    expires_at: DatetimeType = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/licenses/createlicense
    """
    url = urljoin(self.vendors_v2, 'product/generate_license')
    json = {
        'product_id': product_id,
        'allowed_uses': allowed_uses,
    }  # type: PaddleJsonType
    if expires_at:
        json['expires_at'] = validate_date(expires_at, 'expires_at')
    return self.post(url=url, json=json)



import logging
from urllib.parse import urljoin

log = logging.getLogger(__name__)


def list_products(self) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/products/getproducts
    """
    url = urljoin(self.vendors_v2, 'product/get_products')
    return self.post(url=url)



import logging
from urllib.parse import urljoin

log = logging.getLogger(__name__)


def get_order_details(self, checkout_id: str) -> dict:
    """
    https://developer.paddle.com/api-reference/checkout-api/order-information/getorder
    """
    url = urljoin(self.checkout_v1, 'order')
    return self.get(url=url, params={'checkout_id': checkout_id})



import logging
from typing import List
from urllib.parse import urljoin

from .types import PaddleJsonType

log = logging.getLogger(__name__)


def list_plans(self, plan: int = None) -> List[dict]:
    """
    https://developer.paddle.com/api-reference/subscription-api/plans/listplans
    """
    url = urljoin(self.vendors_v2, 'subscription/plans')

    if plan:
        return self.post(url=url, json={'plan': plan})

    return self.post(url=url)


# def get_plan(self, plan: int) -> dict:
#     """
#     https://developer.paddle.com/api-reference/subscription-api/plans/listplans

#     There is no get_plan endpoint in Paddle's API. This is a convenient
#     alias of the list plans endpoint where plan id is required
#     """
#     return self.list_plans(plan=plan)[0]


def create_plan(
    self,
    plan_name: str,
    plan_trial_days: int,
    plan_length: int,
    plan_type: str,
    main_currency_code: str = 'USD',
    initial_price_usd: float = None,
    initial_price_gbp: float = None,
    initial_price_eur: float = None,
    recurring_price_usd: float = None,
    recurring_price_gbp: float = None,
    recurring_price_eur: float = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/subscription-api/plans/createplan

    In the docs all of the price variables are noted as strings except for USD
    which is a number. This is probably an error in the docs.
    """

    url = urljoin(self.vendors_v2, 'subscription/plans_create')

    plan_types = ['day', 'week', 'month', 'year']
    if plan_type not in plan_types:
        raise ValueError('plan_type must be one of {0}'.format(', '.join(plan_types)))  # NOQA: E501
    main_currency_codes = ['USD', 'GBP', 'EUR']
    if main_currency_code not in main_currency_codes:
        raise ValueError('main_currency_code must be one of {0}'.format(', '.join(main_currency_codes)))  # NOQA: E501

    if main_currency_code == 'USD' and initial_price_usd is None:
        raise ValueError('main_currency_code is USD so initial_price_usd must be set')  # NOQA: E501
    if main_currency_code == 'GBP' and initial_price_gbp is None:
        raise ValueError('main_currency_code is USD so initial_price_gbp must be set')  # NOQA: E501
    if main_currency_code == 'USD' and initial_price_eur is None:
        raise ValueError('main_currency_code is USD so initial_price_eur must be set')  # NOQA: E501
    if main_currency_code == 'USD' and recurring_price_usd is None:
        raise ValueError('main_currency_code is USD so recurring_price_usd must be set')  # NOQA: E501
    if main_currency_code == 'GBP' and recurring_price_gbp is None:
        raise ValueError('main_currency_code is USD so recurring_price_gbp must be set')  # NOQA: E501
    if main_currency_code == 'USD' and recurring_price_eur is None:
        raise ValueError('main_currency_code is USD so recurring_price_eur must be set')  # NOQA: E501

    json = {
        'plan_name': plan_name,
        'plan_trial_days': plan_trial_days,
        'plan_length': plan_length,
        'plan_type': plan_type,
        'main_currency_code': main_currency_code,
        'initial_price_usd': initial_price_usd,
        'initial_price_gbp': initial_price_gbp,
        'initial_price_eur': initial_price_eur,
        'recurring_price_usd': recurring_price_usd,
        'recurring_price_gbp': recurring_price_gbp,
        'recurring_price_eur': recurring_price_eur,
    }  # type: PaddleJsonType
    return self.post(url=url, json=json)



import logging
from typing import List
from urllib.parse import urljoin

from .types import PaddleJsonType

log = logging.getLogger(__name__)


def list_subscription_users(
    self,
    subscription_id: int = None,
    plan_id: int = None,
    state: int = None,
    page: int = None,
    results_per_page: int = None,
) -> List[dict]:
    """
    https://developer.paddle.com/api-reference/subscription-api/subscription-users/listusers

    Note: response does not include any information in pages and totals
    """
    url = urljoin(self.vendors_v2, 'subscription/users')

    states = ['active', 'past due', 'trialling', 'paused']
    if state is not None and state not in states:
        raise ValueError('state must be one of {0}'.format(', '.join(states)))

    json = {
        'subscription_id': subscription_id,
        'plan_id': plan_id,
        'state': state,
        'page': page,
        'results_per_page': results_per_page,
    }  # type: PaddleJsonType
    return self.post(url=url, json=json)


# def list_subscriptions(self, **kwargs: dict) -> List[dict]:
#     """
#     https://developer.paddle.com/api-reference/subscription-api/subscription-users/listusers

#     This is a convenient alias for list_subscription_users as while the
#     other name matches the API name, it's not particularly guessable
#     """
#     return self.list_subscription_users(**kwargs)


# def get_subscription(self, subscription_id: int) -> dict:
#     """
#     https://developer.paddle.com/api-reference/subscription-api/subscription-users/listusers

#     There is no get_subscription endpoint in Paddle's API, This is a
#     convenient alias
#     """
#     for subscription in self.list_subscription_users():
#         if subscription['subscription_id'] == subscription_id:
#             return subscription
#     error = 'Unable to find subscription with id {0}'.format(subscription_id)
#     raise ValueError(error)


def cancel_subscription(
    self,
    subscription_id: int,
) -> bool:
    """
    https://developer.paddle.com/api-reference/subscription-api/subscription-users/canceluser
    """
    url = urljoin(self.vendors_v2, 'subscription/users_cancel')
    return self.post(url=url, json={'subscription_id': subscription_id})


def update_subscription(
    self,
    subscription_id: int,
    quantity: int = None,
    currency: str = None,
    recurring_price: float = None,
    bill_immediately: bool = None,
    plan_id: int = None,
    prorate: bool = None,
    keep_modifiers: bool = None,
    passthrough: str = None,
    pause: bool = None
) -> dict:
    """
    https://developer.paddle.com/api-reference/subscription-api/subscription-users/updateuser
    """
    url = urljoin(self.vendors_v2, 'subscription/users/update')

    currency_codes = ['USD', 'GBP', 'EUR']
    if currency and currency not in currency_codes:
        raise ValueError('currency must be one of {0}'.format(', '.join(currency_codes)))  # NOQA: E501

    json = {
        'subscription_id': subscription_id,
        'quantity': quantity,
        'currency': currency,
        'recurring_price': recurring_price,
        'bill_immediately': bill_immediately,
        'plan_id': plan_id,
        'prorate': prorate,
        'keep_modifiers': keep_modifiers,
        'passthrough': passthrough,
        'pause': pause,
    }  # type: PaddleJsonType
    return self.post(url=url, json=json)


# def pause_subscription(self, subscription_id: int) -> dict:
#     """
#     https://developer.paddle.com/api-reference/subscription-api/subscription-users/updateuser

#     There is no resume_subscription endpoint in Paddle's API. This is a
#     convenient alias for update_subscription as no extra data can be sent
#     when pausing/resuming subscriptions
#     """
#     return self.update_subscription(subscription_id=subscription_id, pause=True)  # NOQA: E501


# def resume_subscription(self, subscription_id: int) -> dict:
#     """
#     https://developer.paddle.com/api-reference/subscription-api/subscription-users/updateuser

#     There is no resume_subscription endpoint in Paddle's API. This is a
#     convenient alias for update_subscription as no extra data can be sent
#     when pausing/resuming subscriptions
#     """
#     return self.update_subscription(subscription_id=subscription_id, pause=False)  # NOQA: E501


def preview_subscription_update(
    self,
    subscription_id: int,
    quantity: int = None,
    bill_immediately: bool = None,
    prorate: bool = None,
    plan_id: int = None,
    currency: str = None,
    recurring_price: float = None,
    keep_modifiers: bool = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/subscription-api/subscription-users/previewupdate
    """
    url = urljoin(self.vendors_v2, 'subscription/preview_update')

    currency_codes = ['USD', 'GBP', 'EUR']
    if currency and currency not in currency_codes:
        raise ValueError('currency must be one of {0}'.format(', '.join(currency_codes)))  # NOQA: E501

    json = {
        'subscription_id': subscription_id,
        'quantity': quantity,
        'currency': currency,
        'recurring_price': recurring_price,
        'bill_immediately': bill_immediately,
        'plan_id': plan_id,
        'prorate': prorate,
        'keep_modifiers': keep_modifiers,
    }  # type: PaddleJsonType
    return self.post(url=url, json=json)



import logging
from typing import List
from urllib.parse import urljoin

from .types import DatetimeType, PaddleJsonType
from .validators import validate_date

log = logging.getLogger(__name__)


def list_coupons(self, product_id: int) -> List[dict]:
    """
    https://developer.paddle.com/api-reference/product-api/coupons/listcoupons
    """
    url = urljoin(self.vendors_v2, 'product/list_coupons')
    return self.post(url=url, json={'product_id': product_id})


def create_coupon(
    self,
    coupon_type: str,
    discount_type: str,
    discount_amount: float,
    allowed_uses: int,
    recurring: bool,
    currency: str,
    product_ids: list = None,
    coupon_code: str = None,
    coupon_prefix: str = None,
    num_coupons: int = None,
    description: str = None,
    expires: DatetimeType = None,
    minimum_threshold: int = None,
    group: str = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/coupons/createcoupon

    currency appears to be required:
        Paddle error 134 - The given coupon currency is invalid. The currency must match your balance currency.  # NOQA: E501
    Even though the docs states:
        "Required if discount_amount is flat."
    """

    url = urljoin(self.checkout_v2_1, 'product/create_coupon')

    if coupon_type not in ['product', 'checkout']:
        raise ValueError('coupon_type must be "product" or "checkout"')
    if coupon_type == 'product' and not product_ids:
        raise ValueError('product_ids must be specified if coupon_type is "product"')  # NOQA: E501
    if discount_type not in ['flat', 'percentage']:
        raise ValueError('coupon_type must be "product" or "checkout"')
    if discount_type == 'flat' and not currency:
        raise ValueError('currency must be specified if discount_type is "flat"')  # NOQA: E501
    if coupon_code and (coupon_prefix or num_coupons):
        raise ValueError('coupon_prefix and num_coupons not valid when coupon_code set')  # NOQA: E501

    json = {
        'coupon_type': coupon_type,
        'discount_type': discount_type,
        'discount_amount': discount_amount,
        'allowed_uses': allowed_uses,
        'recurring': 1 if recurring else 0,
    }  # type: PaddleJsonType
    if product_ids:
        products = ','.join([str(int(product)) for product in product_ids])
        json['product_ids'] = products
    if currency:
        if len(currency) != 3:
            raise ValueError('currency must be a 3 letter currency code')
        json['currency'] = currency

    if expires:
        json['expires'] = validate_date(expires, 'expires')

    json['coupon_code'] = coupon_code
    json['coupon_prefix'] = coupon_prefix
    json['num_coupons'] = num_coupons
    json['description'] = description
    json['minimum_threshold'] = minimum_threshold
    json['group'] = group

    return self.post(url=url, json=json)


def delete_coupon(self, coupon_code: str, product_id: int = None) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/coupons/deletecoupon
    """
    url = urljoin(self.vendors_v2, 'product/delete_coupon')
    json = {'coupon_code': coupon_code}  # type: PaddleJsonType
    if product_id:
        json['product_id'] = product_id
    return self.post(url=url, json=json)


def update_coupon(
    self,
    coupon_code: str = None,
    new_coupon_code: str = None,
    group: str = None,
    new_group: str = None,
    product_ids: list = None,
    expires: DatetimeType = None,
    allowed_uses: int = None,
    currency: str = None,
    minimum_threshold: int = None,
    discount_amount: float = None,
    recurring: bool = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/coupons/updatecoupon
    """
    url = urljoin(self.checkout_v2_1, 'product/update_coupon')

    if coupon_code and group:
        raise ValueError('You must specify either coupon_code or group, but not both')  # NOQA: E501

    json = {
        'coupon_code': coupon_code,
        'new_coupon_code': new_coupon_code,
        'group': group,
        'new_group': new_group,
        'allowed_uses': allowed_uses,
        'minimum_threshold': minimum_threshold,
        'discount_amount': discount_amount,
    }  # type: PaddleJsonType
    if product_ids:
        products = ','.join([str(int(product)) for product in product_ids])
        json['product_ids'] = products

    if currency:
        if len(currency) != 3:
            raise ValueError('currency must be a 3 letter currency code')
        json['currency'] = currency

    if expires:
        json['expires'] = validate_date(expires, 'expires')

    if recurring:
        json['recurring'] = 1 if recurring else 0

    return self.post(url=url, json=json)



import logging
from typing import List, Union
from urllib.parse import urljoin

log = logging.getLogger(__name__)


def list_transactions(
    self,
    entity: str,
    entity_id: Union[str, int],
    page: int = None
) -> List[dict]:
    """
    https://developer.paddle.com/api-reference/product-api/transactions/listtransactions
    """
    valid_entities = ['user', 'subscription', 'order', 'checkout', 'product']
    if entity not in valid_entities:
        error = 'entity "{0}" must be one of {1}'.format(
            entity, ",".join(valid_entities)
        )
        raise ValueError(error)

    url = '{entity}/{id}/transactions'.format(entity=entity, id=entity_id)
    url = urljoin(self.vendors_v2, url)

    if page:
        return self.post(url=url, json={'page': page})

    return self.post(url=url)



import logging
from urllib.parse import urljoin

from .types import DatetimeType, PaddleJsonType
from .validators import validate_datetime

log = logging.getLogger(__name__)


def get_webhook_history(
    self,
    page: int = None,
    alerts_per_page: int = None,
    query_head: DatetimeType = None,
    query_tail: DatetimeType = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/alert-api/webhooks/webhooks
    """
    url = urljoin(self.vendors_v2, 'alert/webhooks')

    json = {
        'page': page,
        'alerts_per_page': alerts_per_page,
    }  # type: PaddleJsonType
    if query_head:
        json['query_head'] = validate_datetime(query_head, 'query_head')
    if query_tail:
        json['query_tail'] = validate_datetime(query_tail, 'query_tail')

    return self.post(url=url, json=json)



import logging
from typing import List
from urllib.parse import urljoin

from .constants import countries_requiring_postcode, supported_countries
from .types import DatetimeType, PaddleJsonType
from .validators import validate_date

log = logging.getLogger(__name__)


def create_pay_link(
    self,
    product_id: int = None,
    title: str = None,
    webhook_url: str = None,
    prices: List[str] = None,
    recurring_prices: List[str] = None,
    trial_days: int = None,
    custom_message: str = None,
    coupon_code: str = None,
    discountable: bool = None,
    image_url: str = None,
    return_url: str = None,
    quantity_variable: bool = None,
    quantity: int = None,
    expires: DatetimeType = None,
    affiliates: List[str] = None,
    recurring_affiliate_limit: int = None,
    marketing_consent: str = None,
    customer_email: str = None,
    customer_country: str = None,
    customer_postcode: str = None,
    passthrough: str = None,
    vat_number: str = None,
    vat_company_name: str = None,
    vat_street: str = None,
    vat_city: str = None,
    vat_state: str = None,
    vat_country: str = None,
    vat_postcode: str = None,
) -> dict:
    """
    https://developer.paddle.com/api-reference/product-api/pay-links/createpaylink

    product_id appears to be required:
        Paddle error 108 - Unable to find requested product
    Even though the docs states:
        "If no product_id is set, custom non-subscription product checkouts
        can be generated instead by specifying title, webhook_url and prices."
    https://developer.paddle.com/api-reference/product-api/coupons/createcoupon  # NOQA: E501
    """

    url = urljoin(self.vendors_v2, 'product/generate_license')

    if not product_id:
        if not title:
            raise ValueError('title must be set if product_id is not set')
        if not webhook_url:
            raise ValueError('webhook_url must be set if product_id is not set')  # NOQA: E501
        if recurring_prices:
            raise ValueError('recurring_prices can only be set if product_id is set to a subsciption')  # NOQA: E501
    if customer_country:
        if customer_country not in supported_countries.keys():
            error = 'Country code "{0}" is not valid'.format(customer_country)
            raise ValueError(error)
        if customer_country in countries_requiring_postcode and not customer_postcode:  # NOQA: E501
            error = ('customer_postcode must be set for {0} when '
                     'customer_country is set'.format(vat_country))
            raise ValueError(error)

    if vat_number:
        if not vat_company_name:
            raise ValueError('vat_company_name must be set if vat_number is set')  # NOQA: E501
        if not vat_street:
            raise ValueError('vat_street must be set if vat_number is set')
        if not vat_city:
            raise ValueError('vat_city must be set if vat_number is set')
        if not vat_state:
            raise ValueError('vat_state must be set if vat_number is set')
        if not vat_country:
            raise ValueError('vat_country must be set if vat_number is set')
        if vat_country in countries_requiring_postcode and not vat_postcode:  # NOQA: E501
            error = ('vat_postcode must be set for {0} when '
                     'vat_country is set'.format(vat_country))
            raise ValueError(error)

    json = {
        'product_id': product_id,
        'title': title,
        'webhook_url': webhook_url,
        'prices': prices,
        'recurring_prices': recurring_prices,
        'trial_days': trial_days,
        'custom_message': custom_message,
        'coupon_code': coupon_code,
        'discountable': 1 if discountable is True else 0,
        'image_url': image_url,
        'return_url': return_url,
        'quantity_variable': 1 if quantity_variable is True else 0,
        'quantity': quantity,
        'affiliates': affiliates,
        'recurring_affiliate_limit': recurring_affiliate_limit,
        'marketing_consent': marketing_consent,
        'customer_email': customer_email,
        'customer_country': customer_country,
        'customer_postcode': customer_postcode,
        'passthrough': passthrough,
        'vat_number': vat_number,
        'vat_company_name': vat_company_name,
        'vat_street': vat_street,
        'vat_city': vat_city,
        'vat_state': vat_state,
        'vat_country': vat_country,
        'vat_postcode': vat_postcode,
    }  # type: PaddleJsonType
    if expires:
        json['expires'] = validate_date(expires, 'expires')

    return self.post(url=url, json=json)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paddle']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.23.0,<3.0.0']

setup_kwargs = {
    'name': 'paddle-python',
    'version': '0.5.1',
    'description': 'Python wrapper around the Paddle.com API',
    'long_description': '# Paddle Python\n\nA python (3.5+) wrapper around the [Paddle.com](https://paddle.com/) [API](https://developer.paddle.com/api-reference/intro)\n\n_Note: This is a work in progress, not all of the Paddle endpoints have been implemented yet_\n\n## Quick start\n\n### Installation\n\n```bash\npip install paddle-python\n```\n\n\n### Usage\n\nTo use the Paddle API you will need a Paddle Vendor ID and API key which can be found on [Paddle\'s authentication page](https://vendors.paddle.com/authentication)\n\n```python\nfrom paddle import Paddle\n\n\npaddle = Paddle(vendor_id=12345, api_key=\'myapikey\')\npaddle.list_products()\n```\n\nIf `vendor_id` and `api_key` are not passed through when initalising Paddle will fall back and try and use environmental variables called `PADDLE_VENDOR_ID` and `PADDLE_API_KEY`\n```bash\nexport PADDLE_VENDOR_ID=12345\nexport PADDLE_API_KEY="myapikey"\n```\n\n```python\nfrom paddle import Paddle\n\n\npaddle = Paddle()\npaddle.list_products()\n```\n\n\n## Documentation\n\nComing soon. Please see `Working endpoints` below for basic usage.\n\n\n## Contributing\n\nAll contributions are welcome and appreciated. Please see [CONTRIBUTING.md](https://github.com/pyepye/paddle-python/blob/master/CONTRIBUTING.md) for more details including details on how to run tests etc.\n\n\n## Working endpoints\n* [Get Order Details](https://developer.paddle.com/api-reference/checkout-api/order-information/getorder)\n* [Get User History](https://checkout.paddle.com/api/2.0/user/history)\n* [Get Prices](https://developer.paddle.com/api-reference/checkout-api/prices/getprices)\n* [List Coupons](https://developer.paddle.com/api-reference/product-api/coupons/listcoupons)\n* [Create Coupon](https://developer.paddle.com/api-reference/product-api/coupons/createcoupon)\n* [Delete Coupon](https://developer.paddle.com/api-reference/product-api/coupons/deletecoupon)\n* [Update Coupon](https://developer.paddle.com/api-reference/product-api/coupons/updatecoupon)\n* [List Products](https://developer.paddle.com/api-reference/product-api/products/getproducts)\n* [List Transactions](https://developer.paddle.com/api-reference/product-api/transactions/listtransactions)\n* [Refund Payment](https://developer.paddle.com/api-reference/product-api/payments/refundpayment)\n* [List Plans](https://developer.paddle.com/api-reference/subscription-api/plans/listplans)\n* [Create Plan](https://developer.paddle.com/api-reference/subscription-api/plans/createplan)\n* [List Subscription Users](https://developer.paddle.com/api-reference/subscription-api/subscription-users/listusers)\n* [Cancel Subscription](https://developer.paddle.com/api-reference/subscription-api/subscription-users/canceluser)\n* [Update Subscription](https://developer.paddle.com/api-reference/subscription-api/subscription-users/updateuser)\n* [Preview Subscription Update](https://developer.paddle.com/api-reference/subscription-api/subscription-users/previewupdate)\n* [Get Webhook History](https://developer.paddle.com/api-reference/alert-api/webhooks/webhooks)\n\n```python\npaddle.get_order_details(checkout_id=checkout_id)\npaddle.get_user_history(email=email)\npaddle.get_prices(product_ids=[product_id])\npaddle.list_coupons(product_id=product_id)\npaddle.create_coupon(\n    coupon_type=\'product\',\n    discount_type=\'percentage\',\n    discount_amount=50,\n    allowed_uses=1,\n    recurring=False,\n    currency=\'USD\',\n    product_ids=product_ids,\n    coupon_code=\'50%OFF\',\n    description=\'50% off coupon over $10\',\n    expires=expires,\n    minimum_threshold=10,\n    group=\'paddle-python\',\n)\npaddle.delete_coupon(coupon_code=new_coupon_code, product_id=product_id)\npaddle.update_coupon(\n    coupon_code=coupon_code,\n    new_coupon_code=\'40%OFF\',\n    new_group=\'paddle-python-test\',\n    product_ids=[product_id],\n    expires=expires,\n    allowed_uses=1,\n    currency=\'USD\',\n    minimum_threshold=10,\n    discount_amount=40,\n    recurring=True\n)\npaddle.list_products()\npaddle.list_transactions(entity=\'subscription\', entity_id=subscription_id)\npaddle.refund_payment(order_id=order_id, amount=amount, reason=reason)\npaddle.list_plans()\npaddle.create_plan(\n    plan_name=\'plan_name\',\n    plan_trial_days=14,\n    plan_length=1,\n    plan_type=\'month\',\n    main_currency_code=\'USD\',\n    initial_price_usd=50,\n    recurring_price_usd=50,\n)\npaddle.list_subscription_users()\npaddle.cancel_subscription(subscription_id=1234)\npaddle.update_subscription(subscription_id=1234, pause=True)\npaddle.preview_update_subscription(\n    subscription_id=123,\n    bill_immediately=True,\n    quantity=101,\n)\npaddle.get_webhook_history()\n```\n\n\n## Failing Endpoints\n\nThe below endpoints have been implimented but are not working correctly according to the tests. They have been commented out in `paddle/paddle.py` and the tests will skip is the methods do not exist\n\n* [Generate License](https://developer.paddle.com/api-reference/product-api/licenses/createlicense) - `Paddle error 108 - Unable to find requested product`\n* [Create pay link](https://developer.paddle.com/api-reference/product-api/pay-links/createpaylink) -  `Paddle error 108 - Unable to find requested product`\n\n\n## ToDo\n* Fix generate license and create pay link endpoints\n* Paddle API endpoints\n    * [Add Modifier](https://developer.paddle.com/api-reference/subscription-api/modifiers/createmodifier)\n    * [Delete Modifier](https://developer.paddle.com/api-reference/subscription-api/modifiers/deletemodifier)\n    * [List Modifiers](https://developer.paddle.com/api-reference/subscription-api/modifiers/listmodifiers)\n    * [List Payments](https://developer.paddle.com/api-reference/subscription-api/payments/listpayments)\n    * [Reschedule Payment](https://developer.paddle.com/api-reference/subscription-api/payments/updatepayment)\n    * [Create One-off Charge](https://developer.paddle.com/api-reference/subscription-api/one-off-charges/createcharge)\n* Work out if we want to include \'alias\' methods like pause_subscription\n* Get test coverage to 100%\n* Docs (auto docs?)\n* Use `pytest-mock` `Spy` to check params, json, urls etc for test requests\n    * Needed to any tests which skip due to missing data\n* How to deal with the manual cleanup?\n* Pull request template\n* TravisCI?\n* Dependabot\n* Remove double call for exception error message checking - How to get the exception str from `pytest.raises()`? pytest-mock `Spy`?\n* Add pytest warnings to provide direct links to Paddle for bits that need to be cleaned up\n',
    'author': 'Matt Pye',
    'author_email': 'pyematt@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyepye/paddle-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



from datetime import date, datetime

from .types import DatetimeType


def validate_date(value: DatetimeType, field_name: str) -> str:
    date_format = '%Y-%m-%d'
    if isinstance(value, str):
        error_message = '{0} must be a datetime/data object or string in format YYYY-MM-DD'.format(field_name)  # NOQA: E501
        try:
            datetime.strptime(value, date_format)
        except ValueError:
            raise ValueError(error_message)
        else:
            return value
    elif type(value) in [datetime, date]:
        return value.strftime(date_format)

    raise ValueError(error_message)


def validate_datetime(value: DatetimeType, field_name: str) -> str:
    datetime_format = '%Y-%m-%d %H:%M:%S'
    if isinstance(value, str):
        error_message = '{0} must be a datetime object or string in format YYYY-MM-DD HH:MM:SS'.format(field_name)  # NOQA: E501
        try:
            datetime.strptime(value, datetime_format)
        except ValueError:
            raise ValueError(error_message)
        else:
            return value
    elif type(value) in [datetime, date]:
        return value.strftime(datetime_format)

    raise ValueError(error_message)



import logging
import os
import warnings
from urllib.parse import urljoin

import requests

log = logging.getLogger(__name__)


class PaddleException(Exception):

    def __init__(self, error):
        self.code = 'Unknown'
        self.message = str(error)
        if isinstance(error, requests.HTTPError):  # pragma: no cover - Unsure how to trigger a HTTPError here  # NOQA: E501
            self.code = int(error.response.status_code)
            self.message = 'HTTP error {0} - {1}'.format(
                self.code,
                error.response.content.decode('utf-8'),
            )
        elif isinstance(error, dict):
            try:
                self.code = int(error['code'])
                self.message = 'Paddle error {0} - {1}'.format(
                    error['code'],
                    error['message'],
                )
            except KeyError:  # pragma: no cover - Not sure if this is even possible  # NOQA: E501
                pass

    def __str__(self) -> str:
        return self.message


class Paddle():

    def __init__(self, vendor_id: int = None, api_key: str = None):
        if not vendor_id:
            try:
                vendor_id = int(os.environ['PADDLE_VENDOR_ID'])
            except KeyError:
                raise ValueError('Vendor ID not set')
            except ValueError:
                raise ValueError('Vendor ID must be a number')
        if not api_key:
            try:
                api_key = os.environ['PADDLE_API_KEY']
            except KeyError:
                raise ValueError('API key not set')

        self.checkout_v1 = 'https://checkout.paddle.com/api/1.0/'
        self.checkout_v2 = 'https://checkout.paddle.com/api/2.0/'
        self.checkout_v2_1 = 'https://vendors.paddle.com/api/2.1/'
        self.vendors_v2 = 'https://vendors.paddle.com/api/2.0/'
        self.default_url = self.vendors_v2

        self.vendor_id = vendor_id
        self.api_key = api_key
        self.json = {
            'vendor_id': self.vendor_id,
            'vendor_auth_code': self.api_key,
        }

        self.url_warning = (
            'Paddle recieved a relative URL so it will attempt to join it to '
            '{0} as it is the Paddle URL with the most endpoints. The full '
            'URL that will be used is: {1} - You should specifiy the full URL '
            'as this default URL may change in the future.'
        )

    def request(
        self,
        url: str,
        method: str = 'GET',
        params: dict = None,
        data: dict = None,
        json: dict = None
    ) -> dict:
        kwargs = {}  # type: dict

        # URL join will remove anything after the host name is the url
        # is prepended with a /
        if not url.startswith('http'):
            if url.startswith('/'):
                url = url[1:]
            url = urljoin(self.default_url, url)
            warning_message = self.url_warning.format(self.default_url, url)
            warnings.warn(warning_message, RuntimeWarning)
        if 'paddle.com/api/' not in url:
            raise ValueError('URL "{0}" does not appear to be a Paddle API URL')  # NOQA: E501
        kwargs['url'] = url

        kwargs['method'] = method.upper()

        if data and json:
            raise ValueError('Please set either data or json not both')
        if kwargs['method'] == 'GET' and (data or json):
            log.warn('GET data/json should not be provided with GET method.')

        if kwargs['method'] in ['POST', 'PUT', 'PATCH']:
            kwargs['json'] = {}
            if data:
                kwargs['json'] = data
            if json:
                kwargs['json'] = json
            kwargs['json'] = {k: v for k, v in kwargs['json'].items() if v is not None}  # NOQA: E501
            kwargs['json'].update(self.json)

        if params:
            kwargs['params'] = params

        response = requests.request(**kwargs)
        try:
            response.raise_for_status()
        except requests.HTTPError as e:  # pragma: no cover - Unsure how to trigger a HTTPError here  # NOQA: E501
            raise PaddleException(e)

        response_json = response.json()  # type: dict

        if 'error' in response_json:
            raise PaddleException(response_json['error'])
        # API v1 does not include success
        if 'success' in response_json and not response_json['success']:  # pragma: no cover - Not sure if this is even possible  # NOQA: E501
            raise PaddleException(response_json)

        if 'response' in response_json and response_json['response'] is not None:  # NOQA: E501
            return response_json['response']

        if 'message' in response_json and response_json['message'] is not None:
            return response_json['message']

        # Response is only {"success": True}
        if len(response_json.keys()) == 1 and 'success' in response_json:
            return response_json['success']

        return response_json

    def get(self, url, **kwargs):
        kwargs['url'] = url
        kwargs['method'] = 'GET'
        return self.request(**kwargs)

    def post(self, url, **kwargs):
        kwargs['url'] = url
        kwargs['method'] = 'POST'
        return self.request(**kwargs)

    from ._order_information import get_order_details

    from ._user_history import get_user_history

    from ._prices import get_prices

    from ._coupons import list_coupons
    from ._coupons import create_coupon
    from ._coupons import delete_coupon
    from ._coupons import update_coupon

    from ._products import list_products

    # from ._licenses import generate_license

    # from ._pay_links import create_pay_link

    from ._transactions import list_transactions

    from ._payments import refund_payment

    from ._plans import list_plans
    # from ._plans import get_plan
    from ._plans import create_plan

    from ._subscription_users import list_subscription_users
    from ._subscription_users import cancel_subscription
    from ._subscription_users import update_subscription
    from ._subscription_users import preview_subscription_update
    # Alias to better match update_subscription
    from ._subscription_users import preview_subscription_update as preview_update_subscription  # NOQA: E501

    from ._webhooks import get_webhook_history
